# This code is copyrighted by iuveno AG in June, 2001.
#
# It is Open Source. See the LICENSE.txt for more details!
#
# written by Stephan Richter
# Last updated: July 3, 2001

from AccessControl        import getSecurityManager
from OFS.ObjectManager    import ObjectManager
from Globals              import DTMLFile
from SubObjectTypeSupport import SubObjectTypeSupport


class OrderedObjectManager(SubObjectTypeSupport, ObjectManager):
    '''OrderedObjectManager class - This is an extension to the regular ObjectManager. It saves the objects
       in order and lets you change the order of the contained objects. This is particular helpful, if the
       order does not depend on object attributes, but is totally user-specific.'''
    
    meta_type  ='Ordered Object Manager'

    __ac_permissions__ =  ( ( ('View', ('manage_move_object_up', 'manage_move_object_down',
                                        'manage_move_object_to_top', 'manage_move_object_to_bottom')), )
                            + SubObjectTypeSupport.__ac_permissions__ )

    manage_main = DTMLFile('dtml/main', globals())
    

    def get_object_position(self, id):
        ids = map(lambda obj: obj['id'], self._objects)
        if id in ids:
            return ids.index(id)
        else:
            # If the object was not found, throw an error. Yeah, that is good.
            raise 'ObjectNotFound', 'The object with the id \'%s\' does not exist.' %id


    def move_object_to_position(self, id, position):
        old_position = self.get_object_position(id)
        object = self._objects[old_position]
        # Make sure the new position makes sense and is valid 
        if (old_position == position or position > len(self._objects) or position < 0):
            return 0

        objects = list(self._objects)
        # now delete the entry ...
        objects = filter(lambda obj, id=id: obj['id']!=id, objects) 
        # ... and now add it again
        objects.insert(position, object) 
        self._objects = tuple(objects)

        return 1
        

    def move_object_up(self, id):
        return self.move_object_to_position(id, self.get_object_position(id)-1)

    
    def move_object_down(self, id):
        return self.move_object_to_position(id, self.get_object_position(id)+1)


    def move_object_to_top(self, id):
        return self.move_object_to_position(id, 0)


    def move_object_to_bottom(self, id):
        return self.move_object_to_position(id, len(self._objects))


    def manage_move_object_up(self, id, REQUEST=None):
        '''manage_move_object_up(self, id, REQUEST=None) --> move the object up in the list;
           return HTML or None'''
        self.move_object_up(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_down(self, id, REQUEST=None):
        '''manage_move_object_down(self, id, REQUEST=None) --> move the object down in the list;
           return HTML or None'''
        self.move_object_down(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_to_top(self, id, REQUEST=None):
        '''manage_move_object_to_top(self, id, REQUEST=None) --> move the object to the top of the list;
           return HTML or None'''
        self.move_object_to_top(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_to_bottom(self, id, REQUEST=None):
        '''manage_move_object_to_bottom(self, id, REQUEST=None) --> move the object to the bottom of the list;
           return HTML or None'''
        self.move_object_to_bottom(id)
        if REQUEST:
            return self.manage_main(self, REQUEST)


    _old_manage_renameObject = ObjectManager.inheritedAttribute('manage_renameObject')
    def manage_renameObject(self, id, new_id, REQUEST=None):
        """Rename a particular sub-object"""
        # Since manage_renameObject uses _setObject manually, we have to take care of the order
        # after it is done. Geez, I guess it is not possible to do it differently. PLEASE submit
        # a patch, if you can come up with something better!
        position = self.get_object_position(id)
        result = self._old_manage_renameObject(id, new_id, REQUEST)
        self.move_object_to_position(new_id, position)

        return result


    _old_setObject = ObjectManager._setObject
    def _setObject(self,id,object,roles=None,user=None, set_owner=1, position=None):
        self._old_setObject(id, object, roles, user, set_owner)

        # Now, let's get the position right.
        if position is None:
            position = len(self._objects)
        self.move_object_to_position(id, position)
